#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// En-tête SVG
#define SVG_HEADER "<?xml version=\"1.0\"?>\n<svg width=\"500\" height=\"200\" viewBox=\"-8 -8 516 216\" xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\">\n"
// Pied de page SVG
#define SVG_FOOTER "</svg>\n"

// Fonction pour afficher un message d'erreur et quitter le programme
void print_error_and_exit(const char *message_erreur, const char *manuel_utilisateur) {
    fprintf(stderr, "erreur: %s\n%s\n", message_erreur, manuel_utilisateur);
    exit(EXIT_FAILURE);
}

// Fonction pour afficher le manuel utilisateur
void print_user_manual() {
    printf("motzkin - Générer un SVG pour les chemins Motzkin\n");
    printf("Utilisation : ./motzkin MOT\n");
    printf("  MOT : Séquence de lettres {u, d, r}\n");
}

int main(int argc, char *argv[]) {
    // Manuel utilisateur en cas d'erreur
    const char *manuel_utilisateur = "Pour plus d'informations, consultez le manuel utilisateur en utilisant './motzkin --help'.";
    // Message d'erreur en cas d'argument invalide
    const char *erreur_argument_invalide = "Nombre incorrect d'arguments.";

    // Vérification du nombre d'arguments
    if (argc != 2) {
        print_error_and_exit(erreur_argument_invalide, manuel_utilisateur);
    }

    // Récupération de la séquence de lettres spécifiée par l'utilisateur
    const char *mot = argv[1];
    const int longueur_mot = strlen(mot);

    // Vérification de la longueur de la séquence
    if (longueur_mot > 40) {
        print_error_and_exit("Le chemin ne peut pas être plus large que 40.", manuel_utilisateur);
    }

    // Affichage de l'en-tête SVG
    printf("%s", SVG_HEADER);

    // Affichage des lignes de la grille (similaire à votre exemple)
    printf("<g id=\"grid\">\n");
    for (int i = 0; i <= longueur_mot; ++i) {
        printf("<line x1=\"%d\" y1=\"0\" x2=\"%d\" y2=\"200\" stroke=\"black\" stroke-width=\"1\" stroke-dasharray=\"4 2\"/>\n", i * 100, i * 100);
    }
    printf("</g>\n");

    // Affichage des étapes du chemin
    printf("<g id=\"steps\">\n");

    // Variables pour suivre la position actuelle
    int x = 0;
    int y = 0;

    // Affichage des étapes
    for (int i = 0; i < longueur_mot; ++i) {
        switch (mot[i]) {
            case 'u':
                y += 100;
                break;
            case 'd':
                y -= 100;
                break;
            case 'r':
                x += 100;
                break;
            default:
                print_error_and_exit("Étape non reconnue", manuel_utilisateur);
        }

        // Vérification si le chemin descend en dessous de l'axe horizontal
        if (y < 0) {
            print_error_and_exit("Le chemin ne peut pas descendre en dessous de l'axe horizontal.", manuel_utilisateur);
        }

        // Affichage de la ligne SVG pour l'étape actuelle
        printf("<line x1=\"%d\" y1=\"%d\" x2=\"%d\" y2=\"%d\" stroke=\"red\" stroke-width=\"3\" stroke-dasharray=\"none\"/>\n", x - 100, y - 100, x, y);
    }

    // Vérification si le chemin se termine au-dessus de l'axe horizontal
    if (y > 0) {
        print_error_and_exit("Le chemin ne peut pas se terminer au-dessus de l'axe horizontal.", manuel_utilisateur);
    }

    printf("</g>\n");

    // Affichage des points d'arrêt
    printf("<g id=\"points\">\n");
    for (int i = 0; i <= longueur_mot; ++i) {
        printf("<circle cx=\"%d\" cy=\"0\" r=\"7\" fill=\"blue\"/>\n", i * 100);
    }
    printf("</g>\n");

    // Affichage du pied de page SVG
    printf("%s", SVG_FOOTER);

    return 0;
}
